@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,6 @@
 */
 
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
@@ -38,18 +37,13 @@ int Block::area() const throw()
   }
 
 
-int Block::id( int row, int col ) const throw()
-  { return _blockmap->id( row, col ); }
-
-
 bool Block::compare_id( int id, bool recursive ) const throw()
   {
   if( id == _id ) return true;
   if( recursive )
     {
-    std::list< Block >::const_iterator p = _block_list.begin();
-    for( ; p != _block_list.end(); ++p )
-      if( p->compare_id( id, recursive ) ) return true;
+    for( int i = 0; i < blocks(); ++i )
+      if( _block_vector[i].compare_id( id, recursive ) ) return true;
     }
   return false;
   }
@@ -378,8 +372,8 @@ void Block::print( FILE * outfile, int sp ) const throw()
 
   if( sp >= 0 )
     {
-    std::list< Block >::const_iterator p = _block_list.begin();
-    for( ; p != _block_list.end(); ++p )
+    std::vector< Block >::const_iterator p = _block_vector.begin();
+    for( ; p != _block_vector.end(); ++p )
       p->print( outfile, sp + 8 );
     }
   }
