@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@ class Blockmap;
 
 class Block : public Rectangle
   {
-  std::list< Block > _block_list; // list of blocks nested into this one
+  std::vector< Block > _block_vector; // vector of blocks nested into this one
   const Blockmap * _blockmap;	// Blockmap to which this block belongs
   int _id;			// block identification number
 				// positive for block of black dots
@@ -29,13 +29,13 @@ public:
   Block( const Rectangle & r, const Blockmap & b, int id ) throw()
     : Rectangle( r ), _blockmap( &b ), _id( id ) {}
 
-  const std::list< Block > & block_list() const throw() { return _block_list; }
+  const std::vector< Block > & block_vector() const throw()
+    { return _block_vector; }
   const Blockmap * blockmap() const throw() { return _blockmap; }
 
   int area() const throw();
   int id() const throw() { return _id; }
-  int id( int row, int col ) const throw();
-  int blocks() const throw() { return _block_list.size(); }
+  int blocks() const throw() { return _block_vector.size(); }
   bool compare_id( int id, bool recursive ) const throw();
 
   bool escape_left  ( int row, int col ) const throw();
@@ -54,5 +54,5 @@ public:
 
   void print( FILE * outfile, int sp ) const throw();
 
-  friend void hierarchize_blocks( std::list< Block > & block_list ) throw();
+  friend void hierarchize_blocks( std::vector< Block > & block_vector ) throw();
   };
