@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -24,17 +24,17 @@ class Blockmap
   int _height;
   int _width;
   std::vector< std::vector< int > > data;
+  std::vector< Block > _block_vector;
 
-  // List of clusters of black bits (Blocks), possibly containing
-  // one or more blocks of white bits (holes).
-  // The blocks are (more or less) ordered by position from top to bottom,
-  // and then from left to right.
-  std::list< Block > _block_list;
+  int generate_black_id() { static int id = 0; return ++id; }
+  int generate_white_id() { static int id = 0; return --id; }
 
-  Block * create_block( const Rectangle & r, int id ) throw();
+  void create_block( const Rectangle & r, int id ) throw();
   void delete_block( int id ) throw();
-  Block * join_blocks( int id1, int id2 ) throw();
-  Block * add_point_to_block( int row, int col, int id ) throw();
+  void join_blocks( int id1, int id2 ) throw();
+  void add_point_to_block( int row, int col, int id ) throw();
+
+  void delete_wide_blocks() throw();
 
 public:
   Blockmap( const Bitmap & source, int rindex, int debug_level ) throw();
@@ -42,7 +42,9 @@ public:
   int height() const throw() { return _height; }
   int width()  const throw() { return _width;  }
   int id( int row, int col ) const throw() { return data[row][col]; }
-  const std::list< Block > & block_list() const throw() { return _block_list; }
+  const std::vector< Block > & block_vector() const throw()
+    { return _block_vector; }
+  int blocks() const throw() { return _block_vector.size(); }
 
   void print( FILE * outfile, int debug_level ) const throw();
   };
