@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,11 @@
 */
 
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
+#include "blockmap.h"
 #include "character.h"
 #include "iso_8859_1.h"
 
@@ -30,8 +30,7 @@
 int Character::area() const throw()
   {
   int area = 0;
-  std::list< Block >::const_iterator p = _block_list.begin();
-  for( ; p != _block_list.end(); ++p ) area += p->area();
+  for( int i = 0; i < blocks(); ++i ) area += _block_vector[i].area();
   return area;
   }
 
@@ -41,11 +40,9 @@ void Character::add_block( const Block & b ) throw()
   if( _blockmap != b.blockmap() )
     Ocrad::internal_error( "can't add block to character of different blockmaps.\n" );
   add_rectangle( b );
-  if( _block_list.size() == 0 ) { _block_list.push_back( b ); return; }
-  std::list< Block >::iterator p = _block_list.begin();
-  for( ; p != _block_list.end(); ++p )
-    if( b.vcenter() >= p->vcenter() ) break;
-  _block_list.insert( p, b );
+  int i = 0;
+  while( i < blocks() && b.vcenter() <= _block_vector[i].vcenter() ) ++i;
+  _block_vector.insert( _block_vector.begin() + i, b );
   }
 
 
@@ -79,7 +76,6 @@ void Character::swap_guesses( int i, int j ) throw()
   {
   if( i < 0 || i >= guesses() || j < 0 || j >= guesses() )
     Ocrad::internal_error( "swap_guesses, index out of bounds" );
-//  Guess tmp = _guess[i]; _guess[i] = _guess[j]; _guess[j] = tmp;
   unsigned char ch = _guess[i].ch;
   _guess[i].ch = _guess[j].ch; _guess[j].ch = ch;
   }
@@ -105,8 +101,7 @@ void Character::join( Character & c ) throw()
   {
   if( _blockmap != c._blockmap )
     Ocrad::internal_error( "can't join characters of different blockmaps.\n" );
-  std::list< Block >::const_iterator p = c._block_list.begin();
-  for( ; p != c._block_list.end(); ++p ) add_block( *p );
+  for( int i = 0; i < c.blocks(); ++i ) add_block( c._block_vector[i] );
   }
 
 
@@ -123,21 +118,25 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
     }
   if( graph )
     {
-    const Block * b = ( _block_list.size() ) ? &_block_list.front() : 0;
-    std::fprintf( outfile, "height = %d, width = %d, vcenter = %d, hcenter = %d, black area = %d%%\n",
-                  height(), width(), vcenter(), hcenter(),
+    std::fprintf( outfile, "width = %d, height = %d, hcenter = %d, vcenter = %d, black area = %d%%\n",
+                  width(), height(), hcenter(), vcenter(),
                   ( area() * 100 ) / ( height() * width() ) );
     for( int row = top(); row <= bottom(); ++row)
       {
       for( int col = left(); col <= right(); ++col)
         {
-        char ch = ( row == vcenter() && col == hcenter() ) ? '-' : '';
-        if( b && b->id( row, col ) > 0 )
+        char ch = ( row == vcenter() && col == hcenter() ) ? '+' : '';
+        int id = _blockmap->id( row, col );
+        if( id != 0 )
           {
-          std::list< Block >::const_iterator p = _block_list.begin();
-          for( ; p != _block_list.end(); ++p )
-            if( p->compare_id( b->id( row, col ), recursive ) )
-              { ch = (ch == '-') ? '+' : 'O'; break; }
+          std::vector< Block >::const_iterator p = _block_vector.begin();
+          for( ; p != _block_vector.end(); ++p )
+            if( p->compare_id( id, recursive ) )
+              {
+              if( id > 0 ) ch = (ch == '+') ? 'C' : 'O';
+              else ch = (ch == '+') ? '=' : '-';
+              break;
+              }
           }
         std::fprintf( outfile, " %c", ch );
         }
@@ -161,11 +160,11 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
 
 void Character::xprint( FILE * exportfile ) const throw()
   {
-  std::fprintf( exportfile, "%d %d %d %d;%d",
+  std::fprintf( exportfile, "%3d %3d %2d %2d; %d",
                 left(), top(), width(), height(), guesses() );
 
   for( int i = 0; i < guesses(); ++i )
-    std::fprintf( exportfile, ",'%c'%d", _guess[i].ch, _guess[i].value );
+    std::fprintf( exportfile, ", '%c'%d", _guess[i].ch, _guess[i].value );
 
   std::fputs( "\n", exportfile );
   }
