@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@ class Character : public Rectangle
     Guess( unsigned char c, int v ) throw() : ch( c ), value( v ) {}
     };
 
-  std::list< Block > _block_list; // the blocks forming this Character
+  std::vector< Block > _block_vector; // the blocks forming this Character
   std::vector< Guess > _guess;	// vector of possible char codes
 				// and their associated values
   const Blockmap * _blockmap;	// Blockmap to which this character belongs
@@ -40,21 +40,22 @@ class Character : public Rectangle
 public:
   Character( const Block & b ) throw()
     : Rectangle( b ), _blockmap( b.blockmap() )
-    { _block_list.push_back( b ); }
+    { _block_vector.push_back( b ); }
 
   Character( const Rectangle & r, const Blockmap & b ) throw()
     : Rectangle( r ), _blockmap( &b ) {}
 
   int area() const throw();
-  const std::list< Block > & block_list() const throw() { return _block_list; }
+  const std::vector< Block > & block_vector() const throw()
+    { return _block_vector; }
 //  const std::vector< unsigned char, int > & guess_vector() const throw()
 //    { return _guess; }
   const Blockmap * blockmap() const throw() { return _blockmap; }
 
   void add_block( const Block & b ) throw();
   void only_block( const Block & b ) throw()
-    { _block_list.clear(); _block_list.push_back( b ); }
-  int blocks() const throw() { return _block_list.size(); }
+    { _block_vector.clear(); _block_vector.push_back( b ); }
+  int blocks() const throw() { return _block_vector.size(); }
 
   void add_guess( unsigned char ch, int value ) throw();
   void insert_guess( int i, unsigned char ch, int value ) throw();
