@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -16,12 +16,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <algorithm>
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
+#include "blockmap.h"
 #include "profile.h"
 #include "feats.h"
 #include "iso_8859_1.h"
@@ -40,6 +41,7 @@ int Features::hbars() const throw()
   if( _hbars < 0 )
     {
     const Block & b = *_block;
+    const Blockmap & bm = *_blockmap;
     int state = 0, begin = 0, l = 0, r = 0, limit = wp.max() / 2;
     int count[b.height()];
     _hbars = 0;
@@ -51,7 +53,7 @@ int Features::hbars() const throw()
       maxcount = 0;
       for( col = b.left(); col <= b.right(); ++col )
         {
-        if( b.id( row, col ) == b.id() )
+        if( bm.id( row, col ) == b.id() )
           { ++c; x = col; if( col < b.right() ) continue; }
         if( c > maxcount ) { maxcount = c; rt = x; lt = rt - c + 1; } c = 0;
         }
@@ -87,6 +89,7 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
   if( _vbars < 0 )
     {
     const Block & b = *_block;
+    const Blockmap & bm = *_blockmap;
     int state = 0, begin = 0, limit = b.height();
     limit -= ( b.height() < 40 ) ? 3 : b.height() / 10;
     _vbars = 0;
@@ -96,10 +99,10 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
       int c = 0, c2 = 0, count = 0;
       for( int row = b.top() + 1; row < b.bottom(); ++row )
         {
-        if( b.id( row, col ) == b.id() )
+        if( bm.id( row, col ) == b.id() )
           { ++c; if( row < b.bottom() - 1 ) continue; }
-        else if( ( col > b.left() && b.id( row, col - 1 ) == b.id() ) ||
-                 ( col < b.right() && b.id( row, col + 1 ) == b.id() ) )
+        else if( ( col > b.left() && bm.id( row, col - 1 ) == b.id() ) ||
+                 ( col < b.right() && bm.id( row, col + 1 ) == b.id() ) )
           { ++c; ++c2; if( row < b.bottom() - 1 ) continue; }
         if( c > count ) { count = c; } c = 0;
         }
@@ -138,6 +141,7 @@ const std::vector< int > & Features::hscan() const throw()
   if( !hscan_valid )
     {
     const Block & b = *_block;
+    const Blockmap & bm = *_blockmap;
     int state = 0;
     hscan_valid = true; _hscan.resize( b.width() );
 
@@ -146,7 +150,7 @@ const std::vector< int > & Features::hscan() const throw()
       _hscan[col-b.left()] = 0;
       for( int row = b.top(); row <= b.bottom(); ++row )
         {
-        bool black = ( b.id( row, col ) == b.id() );
+        bool black = ( bm.id( row, col ) == b.id() );
         switch( state )
           {
           case 0: if( black ) state = 1; break;
@@ -166,6 +170,7 @@ const std::vector< int > & Features::vscan() const throw()
   if( !vscan_valid )
     {
     const Block & b = *_block;
+    const Blockmap & bm = *_blockmap;
     int state = 0;
     vscan_valid = true; _vscan.resize( b.height() );
 
@@ -174,7 +179,7 @@ const std::vector< int > & Features::vscan() const throw()
       _vscan[row-b.top()] = 0;
       for( int col = b.left(); col <= b.right(); ++col )
         {
-        bool black = ( b.id( row, col ) == b.id() );
+        bool black = ( bm.id( row, col ) == b.id() );
         switch( state )
           {
           case 0: if( black ) state = 1; break;
@@ -192,23 +197,34 @@ const std::vector< int > & Features::vscan() const throw()
 // upper and lower gaps are open to the right or to the left
 unsigned char Features::test_235Esz() const throw()
   {
+  static const int csize = 5;
+  static const int ucoff[csize] = { 0, +1, -1, -1, +1 };
+  static const int lcoff[csize] = { 0, +1, -1, +1, -1 };
   const Block & b = *_block;
-  int lrow1 = 0, urow2 = 0, lrow2 = 0, urow3 = 0, col, black_section = 0;
+  const Blockmap & bm = *_blockmap;
+  int lrow1 = 0, urow2 = 0, lrow2 = 0, urow3 = 0;
+  int lcol1 = 0, ucol2 = 0, lcol2 = 0, ucol3 = 0, black_section = 0;
+  int cid = bm.id( b.vcenter(), b.hcenter() );
 
-  for( col = b.hcenter() - 1; col <= b.hcenter() + 1; ++col )
+  for( int i = 0; i < csize; ++i )
     {
+    int ucol = b.hcenter() + ucoff[i];
+    int lcol = b.hcenter() + lcoff[i];
+    if( ucol != lcol &&
+        ( bm.id( b.vcenter(), ucol ) != cid || bm.id( b.vcenter(), lcol ) != cid ) )
+      continue;
     bool prev_black = false;
     for( int row = b.top(); row <= b.bottom(); ++row )
       {
-      bool black = ( b.id( row, col ) == b.id() );
+      int col = ( row <= b.vcenter() ) ? ucol : lcol;
+      bool black = ( bm.id( row, col ) == b.id() );
       if( black )
         { if( !prev_black )
-          { if( ++black_section == 2 ) urow2 = row - 1;
-          else if( black_section == 3 ) urow3 = row - 1; }
-        }
+          { if( ++black_section == 2 ) { urow2 = row - 1; ucol2 = col; }
+          else if( black_section == 3 ) { urow3 = row - 1; ucol3 = col; } } }
       else if( prev_black )
-        { if( black_section == 1 ) lrow1 = row;
-          else if( black_section == 2 ) lrow2 = row; }
+        { if( black_section == 1 ) { lrow1 = row; lcol1 = col; }
+          else if( black_section == 2 ) { lrow2 = row; lcol2 = col; } }
       prev_black = black;
       }
     if( black_section == 3 && lrow1 <= b.vpos(30) && urow3 >= b.vpos(70) )
@@ -217,37 +233,40 @@ unsigned char Features::test_235Esz() const throw()
     }
 
   if( black_section != 3 ) return 0;
-  if( b.escape_left( lrow2, col ) )
+  if( b.escape_left( lrow2, lcol2 ) )
     {
-    if( b.escape_left( urow2, col ) )
+    if( b.escape_left( urow2, ucol2 ) )
       {
-      if( b.escape_bottom( lrow2, col ) && b.escape_top( urow2, col ) ) return '*';
+      if( b.escape_bottom( urow3, ucol3 ) && b.escape_top( lrow1, lcol1 ) )
+        return '*';
       return '3';
       }
-    if( b.escape_right( urow2, col ) )
+    if( b.escape_right( urow2, ucol2 ) )
       {
-      if( b.seek_right( lrow1 + 1, col ) >= b.right() ) return '5';
+      if( b.seek_right( lrow1 + 1, lcol1 ) >= b.right() ) return '5';
       if( urow2 > b.vcenter() + 1 && lrow2 > b.bottom() - (b.height() / 5) &&
-          b.seek_right( urow2 - 1, col ) < b.right() )
+          b.seek_right( urow2 - 1, ucol2 ) < b.right() )
         return ISO_8859_1::SCCEDI;
       return 's';
       }
     }
-  else if( b.escape_right( lrow2, col ) )
+  else if( b.escape_right( lrow2, lcol2 ) )
     {
-    if( b.escape_right( urow2, col ) )
+    if( b.escape_right( urow2, ucol2 ) )
       {
-      if( b.escape_bottom( lrow2, col ) && b.escape_top( urow2, col ) )
+      if( b.escape_bottom( urow3, ucol3 ) && b.escape_top( lrow1, lcol1 ) )
         return '*';
       if( bp.minima( b.height() / 5 ) == 1 )
         {
-        if( b.escape_bottom( urow3, col ) ) return 'F';
+        if( b.escape_bottom( urow3, ucol3 ) )
+          { if( 2 * lp[lp.samples()-2] > rp[rp.samples()-2] ) return '';
+          return 'F'; }
         if( b.escape_top( urow2, b.left() ) &&
             !b.escape_top( lrow2, b.left() ) ) return 'f';
         else return 'E';
         }
       }
-    if( b.escape_left( urow2, col ) )
+    if( b.escape_left( urow2, ucol2 ) )
       {
       if( tp.isflat() ) return 'z';
       if( ( tp.isconvex() || ( tp.ispit() && !bp.ispit() ) ) &&
@@ -270,7 +289,7 @@ unsigned char Features::test_49ARegpq() const throw()
   if( bp.minima( b.height() / 10 ) == 2 && bp.isctip() )
     { if( tp.isvpit() ) return 'A'; else return 'R'; }
 
-  const Block & h = b.block_list().front();		// hole
+  const Block & h = b.block_vector().front();		// hole
   int col = h.hcenter();
   int row = b.seek_bottom( h.bottom(), col, false ) + 1;
   if( row >= b.vpos(90) )
@@ -279,7 +298,8 @@ unsigned char Features::test_49ARegpq() const throw()
 
   if( b.escape_right( row, col ) )
     {
-    if( lp.ispit() && b.seek_bottom( row, h.right() ) < b.bottom() )
+    if( ( lp.ispit() && b.seek_bottom( row, h.right() ) < b.bottom() ) ||
+        ( lp.isconvex() && b.seek_bottom( row, h.hcenter() ) < b.bottom() ) )
       return 'e';
     else return 'p';
     }
@@ -294,8 +314,8 @@ unsigned char Features::test_49ARegpq() const throw()
     if( rp.isconvex() && tp.isconvex() && rp.minima() == 1 )
       return '9';
     int hdiff;
-    if( b.top_hook( &hdiff ) && hdiff > 0 ) return 's';
-    if( b.bottom_hook( &hdiff ) && hdiff > 0 ) return 'g';
+    if( b.bottom_hook( &hdiff ) && hdiff > 0 )
+      { if( rp.isctip() ) return 's'; return 'g'; }
     if( row > b.vpos(80) ) return 'Q';
     if( b.seek_bottom( row, col ) < b.bottom() &&
         rp.increasing( row - b.top() ) ) return 'g';
@@ -308,7 +328,7 @@ unsigned char Features::test_49ARegpq() const throw()
 unsigned char Features::test_4ADQao() const throw()
   {
   const Block & b = *_block;
-  const Block & h = b.block_list().front();
+  const Block & h = b.block_vector().front();
 
   if( lp.isflats() && !rp.isflats() ) return 'D';
   if( !rp.isconvex() )
@@ -321,8 +341,8 @@ unsigned char Features::test_4ADQao() const throw()
           b.seek_bottom( row, h.hcenter() ) < b.bottom() )
         return ISO_8859_1::SEACUTE;
       row = ( row + b.seek_bottom( row, h.hcenter() ) ) / 2;
-      if( !lp.isflats() && b.seek_left( row, h.hcenter() ) <= b.left() )
-        return '4';
+      if( row < b.bottom() - 1 && !lp.isflats() &&
+          b.seek_left( row, h.hcenter() ) <= b.left() ) return '4';
       }
     if( b.width() > 3 * h.width() )
       {
@@ -350,21 +370,26 @@ unsigned char Features::test_4ADQao() const throw()
 unsigned char Features::test_6abd() const throw()
   {
   const Block & b = *_block;
-  const Block & h = b.block_list().front();		// hole
+  const Block & h = b.block_vector().front();		// hole
 
   int col = h.hcenter();
   int row = b.seek_top( h.top(), col, false ) - 1;
   if( row <= b.top() )
-    { col = h.right(); row = b.seek_top( h.top(), col, false ) - 1; }
+    {
+    col = h.right(); if( b.right() - h.right() > h.width() ) ++col;
+    row = b.seek_top( h.top(), col, false ) - 1;
+    }
   if( row <= b.top() ) return 0;
   int rcol = ( b.right() + h.right() ) / 2;
   int urow = h.top() - ( b.bottom() - h.bottom() );
-  bool oacute1 = ( b.seek_right( urow - 1, h.right() ) >= b.right() );
+  bool oacute1 = ( ( b.seek_right( urow - 1, h.right() ) >= b.right() ) ||
+                   ( b.seek_right( row, col ) >= b.right() ) );
 
   if( b.escape_right( row, col ) )
     {
-    bool oacute2 = ( b.seek_right( urow, b.left() - 1 ) >
-                     b.seek_right( h.top(), b.left() - 1 ) + 1 );
+    int c = lp[urow-b.top()];
+    bool oacute2 = ( c > lp[h.top()-b.top()] + 1 &&
+                     tp[std::min(c-1,b.width()/4)] + b.top() >= urow );
     if( oacute1 && oacute2 )
       {
       bool oacute3 = ( b.seek_left( b.top() + 1, b.right() + 1 ) >= rcol ||
@@ -393,6 +418,7 @@ unsigned char Features::test_6abd() const throw()
 
   if( b.width() > 3 * h.width() && h.top() < b.vcenter() &&
       vscan()[b.vcenter() - b.top()] == 3 && !lp.isconvex() ) return 'm';
+  int hdiff; if( b.top_hook( &hdiff ) && hdiff > 0 ) return 's';
   return 0;
   }
 
@@ -430,7 +456,9 @@ unsigned char Features::test_CEFIJLlT() const throw()
         if( hbar(0).top() <= b.top() + 1 ) return 'T';
         if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 ) return '+';
         }
-      if( hbars() == 2 && hbar(0).top() <= b.top() + 1 &&
+      if( hbars() == 2 && 4 * hbar(0).height() <= b.height() &&
+          hbar(0).top() <= b.top() + 1 &&
+          hbar(1).bottom() >= b.bottom() - 1 &&
           3 * hbar(0).width() > 4 * hbar(1).width() )
         return 'T';
       }
@@ -443,10 +471,12 @@ unsigned char Features::test_CEFIJLlT() const throw()
       {
       if( hbars() == 3 && hbar(0).top() <= b.top() + 1 &&
           hbar(0).width() + 1 >= hbar(1).width() &&
+          2 * hbar(1).width() >= 3 * vbar(0).width() &&
           vbar(0).h_overlaps( hbar(1) ) )
         {
         if( Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
-            hbar(2).width() >= hbar(1).width() )
+            hbar(2).width() >= hbar(1).width() &&
+            hbar(0).left() <= hbar(1).left() + 1 )
           return 'E';
         if( hbar(0).width() > hbar(2).width() &&
             hbar(1).includes_vcenter( b ) )
@@ -456,6 +486,7 @@ unsigned char Features::test_CEFIJLlT() const throw()
         {
         if( hbar(0).top() <= b.top() + 1 &&
             hbar(0).width() + 1 >= hbar(1).width() &&
+            2 * hbar(1).width() >= 3 * vbar(0).width() &&
             hbar(1).includes_vcenter( b ) )
           return 'F';
         if( hbar(1).bottom() >= b.bottom() - 1 &&
@@ -495,11 +526,12 @@ unsigned char Features::test_CEFIJLlT() const throw()
       if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos(50), 10 ) )
         { if( rp.isvpit() ) return '}'; else return ')'; }
       if( 2 * b.height() > 3 * b.width() )
-        if( vbar(0).right() >= b.hpos( 70 ) ||
-            b.escape_top( b.vpos( 75 ), vbar(0).right() + 1 ) )
-          for( int i = vbar(0).left() - 1; i > b.left(); --i )
-            if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
-                bp[i-b.left()] <= 1 ) return 'l';
+        if( rp.isflats() || rp.minima() == 1 )
+          if( vbar(0).right() >= b.hpos( 70 ) ||
+              b.escape_top( b.vpos( 75 ), vbar(0).right() + 1 ) )
+            for( int i = vbar(0).left() - 1; i > b.left(); --i )
+              if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
+                  bp[i-b.left()] <= 1 ) return 'l';
       }
     if( vbar(0).left() <= b.left() + 1 && b.height() > 2 * b.width() &&
         rp.istip() )
@@ -539,7 +571,8 @@ unsigned char Features::test_c() const throw()
   if( lp.isconvex() )
     {
     if( b.height() > 2 * b.width() ) return '(';
-    if( b.escape_right( b.vcenter(), b.hcenter() ) ) return 'c';
+    if( b.seek_bottom( b.vcenter(), b.hcenter() ) < b.bottom() &&
+        b.escape_right( b.vcenter(), b.hcenter() ) ) return 'c';
     }
   if( lp.ispit() && bp.ispit() && tp.ispit() &&
       b.escape_right( b.vcenter(), b.hcenter() ) )
@@ -601,7 +634,8 @@ unsigned char Features::test_frst() const throw()
           return 'r';
           }
         }
-      if( b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
+      if( vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
+          b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
         return 'x';
       if( lp.isvpit() ) return 't';
       }
@@ -697,9 +731,10 @@ unsigned char Features::test_HKMNUuvwYy() const throw()
               5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
             ( lg > 1 && lg < rg && !rp.increasing() ) ) return 'Y';
       if( 2 * lg < rg && lp.isctip() ) return 'y';
-      if( b.escape_top( b.vcenter(), b.hcenter() ) ) return 'u';
+      if( b.escape_top( b.vpos(60), b.hcenter() ) ) return 'u';
       if( lg < rg + 1 && !lp.increasing() && rp.increasing() &&
-          tp.minima( b.height()/2 ) == 1 ) return 'y';
+          ( tp.minima( b.height()/2 ) == 1 || lp.imaximum() > b.height()/2 ) )
+        return 'y';
       if( lg > 1 && bp.ispit() && tp.minima( ( 2 * b.height() ) / 5 ) == 2 )
         return 'v';
       if( lg <= 1 && 2 * ( b.width() - rg - lg ) < b.width() &&
@@ -762,7 +797,7 @@ unsigned char Features::test_hknwx() const throw()
   if( Ocrad::similar( b.height(), b.width(), 40 ) && row > b.vcenter() &&
       urow < b.vcenter() && tp.minima( b.height() / 5 ) == 2 &&
       bp.minima( urow + 1 ) == 3 ) return 'w';
-  if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 5 ) == 1 &&
+  if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 4 ) == 1 &&
       ( b.width() > b.height() || Ocrad::similar( b.height(), b.width(), 40 ) ) )
     return 'n';
   return 0;
@@ -772,8 +807,9 @@ unsigned char Features::test_hknwx() const throw()
 bool Features::test_comma() const throw()
   {
   const Block & b = *_block;
+  const Blockmap & bm = *_blockmap;
 
-  if( b.block_list().size() > 0 ) return 0;
+  if( b.block_vector().size() > 0 ) return 0;
   if( 4 * b.height() < 5 * b.width()) return false;
   if( b.height() > 3 * b.width() ) return false;
 
@@ -782,12 +818,12 @@ bool Features::test_comma() const throw()
     int upper_area = 0;
     for( int row = b.top(); row < b.top() + b.width(); ++row )
       for( int col = b.left(); col <= b.right(); ++col )
-        if( b.id( row, col ) == b.id() ) ++upper_area;
+        if( bm.id( row, col ) == b.id() ) ++upper_area;
     if( upper_area < (b.width() - 2) * (b.width() - 2) ) return false;
     int count1 = 0, count2 = 0;
     for( int col = b.left(); col <= b.right(); ++col )
-      { if( b.id( b.top() + 1, col ) == b.id() ) ++count1;
-      if( b.id( b.bottom() - 1, col ) == b.id() ) ++count2; }
+      { if( bm.id( b.top() + 1, col ) == b.id() ) ++count1;
+      if( bm.id( b.bottom() - 1, col ) == b.id() ) ++count2; }
     if( count1 <= count2 ) return false;
     }
   return true;
@@ -807,7 +843,7 @@ unsigned char Features::test_easy( int charbox_vcenter ) const throw()
     if( tp.iminimum() < tp.pos(50) && bp.iminimum() > bp.pos(50) )
       return '`'; else return '\'';
     }
-  if( 2 * b.height() > 3 * b.width() && b.top() >= charbox_vcenter &&
+  if( 2 * b.height() > 3 * wp.max() && b.top() >= charbox_vcenter &&
       bp.minima() == 1 ) return ',';
   return 0;
   }
@@ -853,6 +889,8 @@ unsigned char Features::test_solid( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
   if( b.blocks() ) return 0;
+  const Blockmap & bm = *_blockmap;
+  if( bm.id( b.vcenter(), b.hcenter() ) != b.id() ) return 0;
   int inner_area, inner_space;
 
   if( b.width() >= 3 && b.height() >= 3 )
@@ -863,7 +901,7 @@ unsigned char Features::test_solid( int charbox_vcenter ) const throw()
       {
       int holes = 0;
       for( int col = b.left() + 1; col < b.right(); ++col )
-        { if( b.id( row, col ) == b.id() ) ++inner_area; else ++holes; }
+        { if( bm.id( row, col ) == b.id() ) ++inner_area; else ++holes; }
       if( 5 * holes >= b.width() ) return 0;
       }
     if( inner_area * 100 < inner_space * 75 ) return 0;
@@ -883,17 +921,17 @@ unsigned char Features::test_solid( int charbox_vcenter ) const throw()
       {
       int l = 0, r = 0;
       for( int row = b.vcenter() - 1; row <= b.vcenter() + 1; ++row )
-        { if( b.id( row, b.left() ) == b.id() ) ++l;
-        if( b.id( row, b.right() ) == b.id() ) ++r; }
+        { if( bm.id( row, b.left() ) == b.id() ) ++l;
+        if( bm.id( row, b.right() ) == b.id() ) ++r; }
       if( l <= 1 && r <= 1 )
         {
         bool lt = false, lb = false, rt = false, rb = false;
         for( int row = b.top(); row <= b.top() + 2; ++row )
-          { if( b.id( row, b.left() ) == b.id() ) lt = true;
-          if( b.id( row, b.right() ) == b.id() ) rt = true; }
+          { if( bm.id( row, b.left() ) == b.id() ) lt = true;
+          if( bm.id( row, b.right() ) == b.id() ) rt = true; }
         for( int row = b.bottom() - 2; row <= b.bottom(); ++row )
-          { if( b.id( row, b.left() ) == b.id() ) lb = true;
-          if( b.id( row, b.right() ) == b.id() ) rb = true; }
+          { if( bm.id( row, b.left() ) == b.id() ) lb = true;
+          if( bm.id( row, b.right() ) == b.id() ) rb = true; }
         if( lt && lb && rb ) { if( rt ) return 'I'; else return 'l'; }
         }
       }
