@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,11 @@
 */
 
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
+#include "blockmap.h"
 #include "profile.h"
 
 
@@ -37,6 +37,7 @@ Profile::Profile( const Block & b, type t ) throw()
 void Profile::initialize() throw()
   {
   const Block & b = *_block;
+  const Blockmap & bm = *_blockmap;
 
   valid = true;
   switch( _type )
@@ -46,7 +47,7 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int j = b.left(), row = b.top() + i;
-        while( j <= b.right() && b.id( row, j ) != b.id() ) ++j;
+        while( j <= b.right() && bm.id( row, j ) != b.id() ) ++j;
         data[i] = j - b.left();
         } break;
     case top :
@@ -54,7 +55,7 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int j = b.top(), col = b.left() + i;
-        while( j <= b.bottom() && b.id( j, col ) != b.id() ) ++j;
+        while( j <= b.bottom() && bm.id( j, col ) != b.id() ) ++j;
         data[i] = j - b.top();
         } break;
     case right :
@@ -62,7 +63,7 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int j = b.right(), row = b.top() + i;
-        while( j >= b.left() && b.id( row, j ) != b.id() ) --j;
+        while( j >= b.left() && bm.id( row, j ) != b.id() ) --j;
         data[i] = b.right() - j;
         } break;
     case bottom :
@@ -70,7 +71,7 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int j = b.bottom(), col = b.left() + i;
-        while( j >= b.top() && b.id( j, col ) != b.id() ) --j;
+        while( j >= b.top() && bm.id( j, col ) != b.id() ) --j;
         data[i] = b.bottom() - j;
         } break;
     case height :
@@ -78,8 +79,8 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int u = b.top(), d = b.bottom(), col = b.left() + i;
-        while( u <= d && b.id( u, col ) != b.id() ) ++u;
-        while( u <= d && b.id( d, col ) != b.id() ) --d;
+        while( u <= d && bm.id( u, col ) != b.id() ) ++u;
+        while( u <= d && bm.id( d, col ) != b.id() ) --d;
         data[i] = ( u <= d ) ? d - u + 1 : 0;
         } break;
     case width :
@@ -87,8 +88,8 @@ void Profile::initialize() throw()
       for( int i = 0; i < samples(); ++i )
         {
         int l = b.left(), r = b.right(), row = b.top() + i;
-        while( l <= r && b.id( row, l ) != b.id() ) ++l;
-        while( l <= r && b.id( row, r ) != b.id() ) --r;
+        while( l <= r && bm.id( row, l ) != b.id() ) ++l;
+        while( l <= r && bm.id( row, r ) != b.id() ) --r;
         data[i] = ( l <= r ) ? r - l + 1 : 0;
         } break;
     case zheight :
@@ -98,7 +99,7 @@ void Profile::initialize() throw()
         int col = b.left() + i;
         data[i] = 0;
         for( int j = b.top(); j <= b.bottom(); ++j )
-          if( b.id( j, col ) == b.id() ) ++data[i];
+          if( bm.id( j, col ) == b.id() ) ++data[i];
         } break;
     case zwidth :
       data.resize( b.height() ); _limit = b.width();
@@ -107,7 +108,7 @@ void Profile::initialize() throw()
         int row = b.top() + i;
         data[i] = 0;
         for( int j = b.left(); j <= b.right(); ++j )
-          if( b.id( row, j ) == b.id() ) ++data[i];
+          if( bm.id( row, j ) == b.id() ) ++data[i];
         } break;
     }
   }
