@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,11 @@
 */
 
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
+#include "blockmap.h"
 #include "character.h"
 #include "profile.h"
 #include "feats.h"
@@ -31,9 +31,9 @@
 // First attempt at recognition without relying on context.
 void Character::recognize1( int charbox_vcenter ) throw()
   {
-  if( _block_list.size() == 1 ) recognize11( charbox_vcenter );
-  else if( _block_list.size() == 2 ) recognize12( charbox_vcenter );
-  else if( _block_list.size() == 3 ) recognize13();
+  if( _block_vector.size() == 1 ) recognize11( charbox_vcenter );
+  else if( _block_vector.size() == 2 ) recognize12( charbox_vcenter );
+  else if( _block_vector.size() == 3 ) recognize13();
   }
 
 
@@ -42,10 +42,10 @@ void Character::recognize1( int charbox_vcenter ) throw()
 // #$&'()*+,-./<>@[\]^_`{|}~
 void Character::recognize11( int charbox_vcenter ) throw()
   {
-  const Block & b = _block_list.front();
-  if( b.block_list().size() == 0 ) recognize110( charbox_vcenter );
-  else if( b.block_list().size() == 1 ) recognize111();
-  else if( b.block_list().size() == 2 ) recognize112();
+  const Block & b = _block_vector.front();
+  if( b.block_vector().size() == 0 ) recognize110( charbox_vcenter );
+  else if( b.block_vector().size() == 1 ) recognize111();
+  else if( b.block_vector().size() == 2 ) recognize112();
   }
 
 
@@ -54,7 +54,7 @@ void Character::recognize11( int charbox_vcenter ) throw()
 // '()*+,-./<>@[\]^_`{|}~
 void Character::recognize110( int charbox_vcenter ) throw()
   {
-  const Block & b = _block_list.front();
+  const Block & b = _block_vector.front();
   Features f( b );
   unsigned char ch = f.test_easy( charbox_vcenter );
   if( ch )
@@ -63,6 +63,7 @@ void Character::recognize110( int charbox_vcenter ) throw()
       { add_guess( ch, 1 ); add_guess( '-', 0 ); return; }
     add_guess( ch, 0 ); return;
     }
+  if( b.height() < 6 && b.width() < 6 ) return;
 
   ch = f.test_CEFIJLlT();   if( ch ) { add_guess( ch, 0 ); return; }
   ch = f.test_frst();       if( ch ) { add_guess( ch, 0 ); return; }
@@ -167,8 +168,8 @@ void Character::recognize110( int charbox_vcenter ) throw()
 // 0469ADOPQRabdegopq#
 void Character::recognize111() throw()
   {
-  Block & b = _block_list.front();
-  const Block & h = b.block_list().front();
+  Block & b = _block_vector.front();
+  const Block & h = b.block_vector().front();
   Features f( b );
   int top_delta = h.top() - b.top(), bottom_delta = b.bottom() - h.bottom();
 
@@ -203,7 +204,7 @@ void Character::recognize111() throw()
         if( row <= b.top() || row + 1 >= h.top() ) return;
         Block b1( b, *b.blockmap(), b.id() );
         b.top( row + 1 ); b1.bottom( row );
-        _block_list.push_back( b1 );
+        _block_vector.push_back( b1 );
         }
       }
     }
@@ -214,9 +215,9 @@ void Character::recognize111() throw()
 // 8BQg$&
 void Character::recognize112() throw()
   {
-  const Block & b = _block_list.front();
-  const Block & h1 = b.block_list().front();		// upper hole
-  const Block & h2 = b.block_list().back();		// lower hole
+  const Block & b = _block_vector.front();
+  const Block & h1 = b.block_vector().front();		// upper hole
+  const Block & h2 = b.block_vector().back();		// lower hole
   int a1 = h1.area();
   int a2 = h2.area();
 //  Features f( b );
@@ -239,15 +240,24 @@ void Character::recognize112() throw()
     int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
     int col2 = h2.seek_right( h2.top(), h2.left() - 1 ) + 1;
     if( col1 <= col2 )
-      { if( lp.isconvex() ) add_guess( 'e', 1 ); add_guess( '$', 0 ); return; }
+      {
+      if( lp.isconvex() || lp.ispit() ) add_guess( 'e', 1 );
+      else if( !rp.isctip() ) add_guess( 'a', 1 );
+      add_guess( '$', 0 ); return;
+      }
 
     if( b.hcenter() > h1.hcenter() && b.hcenter() > h2.hcenter() &&
         ( b.hcenter() >= h1.right() || b.hcenter() >= h2.right() ) )
       { add_guess( '&', 0 ); return; }
     for( int row = h1.bottom() + 1; row < h2.top(); ++row )
-      if( b.id( row, hcenter() ) == 0 ) { add_guess( 'g', 0 ); return; }
+      if( _blockmap->id( row, hcenter() ) == 0 )
+        { add_guess( 'g', 0 ); return; }
     if( bp.isconvex() )
-      { if( lp.isconvex() ) add_guess( 'e', 1 ); add_guess( '8', 0 ); return; }
+      {
+      if( lp.ispit() || lp.isconvex() ) add_guess( 'e', 1 );
+      else if( rp.ispit() ) add_guess( 'a', 1 );
+      add_guess( '8', 0 ); return;
+      }
     if( lp.minima() == 2 && rp.minima() == 1 ) { add_guess( 'a', 0 ); return; }
     add_guess( 'B', 1 ); add_guess( 'a', 0 ); return;
     }
@@ -256,6 +266,7 @@ void Character::recognize112() throw()
     if( !h1.v_overlaps( h2 ) ) { add_guess( 'g', 0 ); return; }
     add_guess( 'Q', 0 ); return;
     }
+  if( rp.minima() == 1 ) { add_guess( 'a', 0 ); return; }
   add_guess( '&', 0 );
   }
 
@@ -264,8 +275,8 @@ void Character::recognize112() throw()
 // ij!%:;=?|
 void Character::recognize12( int charbox_vcenter ) throw()
   {
-  const Block & b1 = _block_list.front();		// lower block
-  const Block & b2 = _block_list.back();		// upper block
+  const Block & b1 = _block_vector.front();		// lower block
+  const Block & b2 = _block_vector.back();		// upper block
   int a1 = b1.area();
   int a2 = b2.area();
   Features f1( b1 );
@@ -369,9 +380,9 @@ void Character::recognize12( int charbox_vcenter ) throw()
     }
   if( ch == '-' || ch == '_' )
     {
-    if( b2.block_list().size() == 1 )
+    if( b2.block_vector().size() == 1 )
       {
-      const Block & h = b2.block_list().front();
+      const Block & h = b2.block_vector().front();
       if( Ocrad::similar( h.left() - b2.left(), b2.right() - h.right(), 40 ) )
         { add_guess( ISO_8859_1::MASCORD, 0 ); return; }
       add_guess( ISO_8859_1::FEMIORD, 0 ); return;
@@ -384,7 +395,7 @@ void Character::recognize12( int charbox_vcenter ) throw()
 // %
 void Character::recognize13() throw()
   {
-  const Block & b1 = _block_list.front();
+  const Block & b1 = _block_vector.front();
   Features f1( b1 );
   if( f1.test_solid( vcenter() ) == '.' ) add_guess( ISO_8859_1::DIV, 0 );
   Character c( b1 );
