@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@ Rectangle::Rectangle( int l, int t, int r, int b ) throw()
   _left = l; _top = t; _right = r; _bottom = b;
   }
 
+
 void Rectangle::left( int l ) throw()
   {
   if( l > _right )
@@ -39,6 +40,7 @@ void Rectangle::left( int l ) throw()
   _left = l;
   }
 
+
 void Rectangle::top( int t ) throw()
   {
   if( t > _bottom )
@@ -46,6 +48,7 @@ void Rectangle::top( int t ) throw()
   _top = t;
   }
 
+
 void Rectangle::right( int r ) throw()
   {
   if( r < _left )
@@ -53,6 +56,7 @@ void Rectangle::right( int r ) throw()
   _right = r;
   }
 
+
 void Rectangle::bottom( int b ) throw()
   {
   if( b < _top )
@@ -60,6 +64,7 @@ void Rectangle::bottom( int b ) throw()
   _bottom = b;
   }
 
+
 int Rectangle::hpos( int p ) const throw()
   { return ( ( ( _right - _left ) * p ) / 100 ) + _left; }
 
@@ -74,6 +79,7 @@ void Rectangle::add_point( int row, int col ) throw()
   if( col > _right ) _right = col;   else if( col < _left ) _left = col;
   }
 
+
 void Rectangle::add_rectangle( const Rectangle & r ) throw()
   {
   if( r._left < _left ) _left = r._left;
@@ -82,37 +88,63 @@ void Rectangle::add_rectangle( const Rectangle & r ) throw()
   if( r._bottom > _bottom ) _bottom = r._bottom;
   }
 
+
 bool Rectangle::includes( const Rectangle & r ) const throw()
   {
   return ( _left  <= r._left  && _top    <= r._top &&
            _right >= r._right && _bottom >= r._bottom );
   }
 
+
 bool Rectangle::includes( int row, int col ) const throw()
   {
   return ( _left <= col && _right >= col && _top <= row && _bottom >= row );
   }
 
+
+bool Rectangle::strictly_includes( const Rectangle & r ) const throw()
+  {
+  return ( _left  < r._left  && _top    < r._top &&
+           _right > r._right && _bottom > r._bottom );
+  }
+
+
 bool Rectangle::includes_hcenter( const Rectangle & r ) const throw()
   { return ( _left <= r.hcenter() && _right >= r.hcenter() ); }
 
+
 bool Rectangle::includes_vcenter( const Rectangle & r ) const throw()
   { return ( _top <= r.vcenter() && _bottom >= r.vcenter() ); }
 
+
 bool Rectangle::h_includes( const Rectangle & r ) const throw()
   { return ( _left <= r._left && _right >= r._right ); }
 
+
 bool Rectangle::v_includes( const Rectangle & r ) const throw()
   { return ( _top <= r._top && _bottom >= r._bottom ); }
 
+
 bool Rectangle::h_includes( int col ) const throw()
   { return ( _left <= col && _right >= col ); }
 
+
 bool Rectangle::v_includes( int row ) const throw()
   { return ( _top <= row && _bottom >= row ); }
 
+
 bool Rectangle::h_overlaps( const Rectangle & r ) const throw()
   { return ( _left <= r.right() && _right >= r.left() ); }
 
+
 bool Rectangle::v_overlaps( const Rectangle & r ) const throw()
   { return ( _top <= r.bottom() && _bottom >= r.top() ); }
+
+/*
+bool Rectangle::precedes( const Rectangle & r ) const throw()
+  {
+  if( v_overlaps( r ) ) return ( _left < r._left );
+  if( h_overlaps( r ) ) return ( _top < r._top );
+  return ( _left < r._left && _top < r._top );
+  }
+*/
