@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,6 @@
 */
 
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
@@ -27,14 +26,14 @@
 #include "textblock.h"
 
 
-Textblock::Textblock( const std::list< Block > & block_list ) throw()
+Textblock::Textblock( const std::vector< Block > & block_vector ) throw()
   {
   // First pass. Assign blocks to characters and create lines.
   {
   const Character * prev = 0;
   int current_line = 0;
-  for( std::list< Block >::const_iterator p = block_list.begin();
-       p != block_list.end(); ++p )
+  for( std::vector< Block >::const_iterator p = block_vector.begin();
+       p != block_vector.end(); ++p )
     {
     if( p->height() * p->width() < 4 ) continue;
     if( p->height() > 20 * p->width() ) continue;
@@ -132,7 +131,7 @@ Textblock::Textblock( const std::list< Block > & block_list ) throw()
         int right = c2.left() - 1;
         int bottom = ( c1.bottom() + c2.bottom() ) / 2;
         Character c( Rectangle( left, top, right, bottom ), *c1.blockmap() );
-        c.add_guess( ' ', 100 );
+        c.add_guess( ' ', 0 );
         line1.insert_character( ++i, c );
         }
       }
