@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@ class Textblock
   mutable std::vector< Textline > data;
 
 public:
-  Textblock( const std::list< Block > & block_list ) throw();
+  Textblock( const std::vector< Block > & block_vector ) throw();
 
   int alnum_percent() const throw();
   void add_line() throw() { data.push_back( Textline() ); }
