@@ -1,4 +1,4 @@
-/*  GNU OCRAD - Optical Character Recognition program
+/*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 
 #include <cctype>
 #include <cstdio>
-#include <list>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
@@ -214,28 +213,30 @@ void Textline::recognize2() throw()
   if( characters() == 0 ) return;
 
   // separate merged characters recognized by recognize1
-  for( int i = 0; i < characters(); )
+  for( int i = 0; i < characters(); ++i )
     {
-    const Character & c = character( i );
-    if( c.guesses() > 2 && c.guess( 0 ).ch == 0 && c.blocks() >= 1 )
+    int j = i, guesses = data[j].guesses();
+    if( guesses >= 1 && data[j].guess( 0 ).ch == 0 )
       {
-      int g, left = c.guess( 0 ).value;
-      for( g = 1; g < c.guesses(); ++g )
+      if( guesses >= 3 && data[j].blocks() >= 1 )
         {
-        const Character & c = character( i );
-        const Block & b = c.block_list().front();
-        Rectangle r( left, b.top(), c.guess( g ).value, b.bottom() );
-        Character c1( Block( r, *c.blockmap(), b.id() ) );
-        std::list< Block >::const_iterator p = c.block_list().begin();
-        for( ++p; p != c.block_list().end(); ++p )
-          if( r.includes_hcenter( *p ) ) c1.add_block( *p );
-        c1.add_guess( c.guess( g ).ch, 0 );
-        left = c.guess( g ).value + 1;
-        insert_character( i + g, c1 );
+        int left = data[j].guess( 0 ).value;
+        for( int g = 1; g < guesses; ++g )
+          {
+          const Block & b = data[j].block_vector().front();
+          Rectangle r( left, b.top(), data[j].guess( g ).value, b.bottom() );
+          Character c1( Block( r, *data[j].blockmap(), b.id() ) );
+          std::vector< Block >::const_iterator p = data[j].block_vector().begin();
+          for( ++p; p != data[j].block_vector().end(); ++p )
+            if( r.includes_hcenter( *p ) ) c1.add_block( *p );
+          c1.add_guess( data[j].guess( g ).ch, 0 );
+          left = data[j].guess( g ).value + 1;
+          insert_character( j + g, c1 );
+          }
+        i += guesses - 2;
         }
-      delete_character( i ); i += g - 1;
+      delete_character( j );
       }
-    else ++i;
     }
 
   // try to recognize separately the 2 overlapped blocks of an
@@ -243,11 +244,11 @@ void Textline::recognize2() throw()
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    if( c.guesses() == 0 && c.block_list().size() == 2 &&
-        c.block_list().front().v_overlaps( c.block_list().back() ) )
+    if( c.guesses() == 0 && c.block_vector().size() == 2 &&
+        c.block_vector().front().v_overlaps( c.block_vector().back() ) )
       {
-      Character c1( c.block_list().front() );
-      Character c2( c.block_list().back() );
+      Character c1( c.block_vector().front() );
+      Character c2( c.block_vector().back() );
       c1.recognize1( c.vcenter() );
       c2.recognize1( c.vcenter() );
       if( c1.left() <= c2.left() ) { c = c1; insert_character( i + 1, c2 ); }
@@ -277,7 +278,7 @@ void Textline::recognize2() throw()
           unsigned char ch2 = c2.guess( 0 ).ch;
           if( std::isspace( ch2 ) ) break;
           if( ( std::isalpha( ch2 ) && 4 * c1.height() > 5 * c2.height() ) ||
-              ( std::isupper( ch2 ) && ch2 != 'B' &&
+              ( ( std::isupper( ch2 ) || ch2 == 'l' ) && ch2 != 'B' &&
                 ( c1.height() >= c2.height() ||
                   Ocrad::similar( c1.height(), c2.height(), 10 ) ) ) )
             { c1.insert_guess( 0, std::toupper( ch ), 1 ); break; }
@@ -302,25 +303,6 @@ void Textline::recognize2() throw()
       }
     }
 
-  // transform a small p to a capital p
-  for( int i = characters() - 1; i > 0; --i )
-    {
-    Character & c1 = character( i - 1 );
-    if( c1.guesses() == 1 && c1.guess( 0 ).ch == 'p' )
-      {
-      Character & c2 = character( i );
-      if( c2.guesses() == 0 || !std::isalnum( c2.guess( 0 ).ch ) ) continue;
-      switch( c2.guess( 0 ).ch )
-        {
-        case 'g': case 'j': case 'p': case 'q': case 'y':
-                  if( c1.bottom() + 2 > c2.bottom() ) continue; break;
-        case 'Q': if( std::abs( c1.top() - c2.top() ) > 2 ) continue; break;
-        default : if( std::abs( c1.bottom() - c2.bottom() ) > 2 ) continue; break;
-        }
-      c1.only_guess( 'P', 0 );
-      }
-    }
-
   // transform small o or u with accent or diaeresis to capital
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
@@ -329,10 +311,10 @@ void Textline::recognize2() throw()
       {
       unsigned char ch = c1.guess( 0 ).ch;
       if( std::isspace( ch ) ) { begin = i + 1 ; continue; }
-      if( ch <= 127 || c1.block_list().size() < 2 ) continue;
+      if( ch <= 127 || c1.block_vector().size() < 2 ) continue;
       int chb = ISO_8859_1::base_letter( ch );
       if( chb != 'o' && chb != 'u' ) continue;
-      const Block & b1 = c1.block_list().front();
+      const Block & b1 = c1.block_vector().front();
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
@@ -356,7 +338,7 @@ void Textline::recognize2() throw()
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.guesses() == 1 )
+    if( c1.guesses() >= 1 )
       {
       unsigned char ch = c1.guess( 0 ).ch;
       if( std::isspace( ch ) ) { begin = i + 1 ; continue; }
@@ -371,7 +353,7 @@ void Textline::recognize2() throw()
           if( std::isdigit( ch2 ) )
             {
             if( Ocrad::similar( c1.height(), c2.height(), 10 ) )
-              c1.insert_guess( 0, (ch == 'l') ? '1' : '0', 1 );
+              c1.insert_guess( 0, (ch == 'l') ? '1' : '0', c1.guess(0).value + 1 );
             break;
             }
           if( ISO_8859_1::isalpha( ch2 ) &&
@@ -381,11 +363,62 @@ void Textline::recognize2() throw()
       }
     }
 
-  // transform a vertical bar into l or I
+  // transform a small p to a capital P
+  for( int i = characters() - 1; i > 0; --i )
+    {
+    Character & c1 = character( i - 1 );
+    if( c1.guesses() == 1 && c1.guess( 0 ).ch == 'p' )
+      {
+      Character & c2 = character( i );
+      if( !c2.guesses() ) continue;
+      unsigned char ch = c2.guess( 0 ).ch;
+      if( !std::isalnum( ch ) && ch != '.' && ch != '|' ) continue;
+      switch( ch )
+        {
+        case 'g': case 'j': case 'p': case 'q': case 'y':
+                  if( c1.bottom() + 2 > c2.bottom() ) continue; break;
+        case 'Q': if( std::abs( c1.top() - c2.top() ) > 2 ) continue; break;
+        default : if( std::abs( c1.bottom() - c2.bottom() ) > 2 ) continue; break;
+        }
+      c1.only_guess( 'P', 0 );
+      }
+    }
+
+  // transform words like 'lO.OOO' into numbers like '10.000'
+  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+    {
+    end = find_space( begin );
+    if( end - begin < 2 ) continue;
+    Character & c1 = character( begin );
+    if( !c1.guesses() ) continue;
+    if( c1.guess(0).ch == 'l' || c1.guess(0).ch == 'O' )
+      {
+      int i = begin + 1;
+      for( ; i < end; ++i )
+        {
+        Character & c = character( i );
+        if( !c.guesses() ) break;
+        unsigned char ch = c.guess(0).ch;
+        if( ch != 'l' && ch != 'O' && ch != '.' && ch != ',' ) break;
+        }
+      if( i >= end )
+        for( i = begin; i < end; ++i )
+        {
+        Character & c = character( i );
+        unsigned char ch = c.guess(0).ch;
+        if( ch == 'l' ) ch = '1'; else if( ch == 'O' ) ch = '0'; else ch = 0;
+        if( ch ) c.insert_guess( 0, ch, c.guess(0).value + 1 );
+        }
+      }
+    }
+
+  // transform a vertical bar into l or I (or a l into an I)
   for( int i = 0; i < characters(); ++i )
     {
     Character & c = character( i );
-    if( c.guesses() == 1 && c.guess( 0 ).ch == '|' )
+    if( c.guesses() != 1 ) continue;
+    unsigned char ch = c.guess( 0 ).ch;
+    if( ch == '|' || ch == 'l' )
       {
       unsigned char lch = 0, rch = 0;
       if( i > 0 && character( i - 1 ).guesses() )
@@ -395,6 +428,7 @@ void Textline::recognize2() throw()
       if( ISO_8859_1::isupper( rch ) &&
           ( !lch || ISO_8859_1::isupper( lch ) || std::isspace( lch ) ) )
         { c.insert_guess( 0, 'I', 1 ); continue; }
+      if( ch == 'l' ) continue;
       if( ISO_8859_1::isalpha( lch ) || ISO_8859_1::isalpha( rch ) )
         { c.insert_guess( 0, 'l', 1 ); continue; }
       if( rch == '|' && ( !lch || std::isspace( lch ) ) &&
@@ -404,30 +438,6 @@ void Textline::recognize2() throw()
       }
     }
 
-  // transform words like 'lOOO' into numbers like '1000'
-  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
-    {
-    end = find_space( begin );
-    if( end - begin < 2 ) continue;
-    Character & c1 = character( begin );
-    if( c1.guesses() >= 1 && c1.guess(0).ch == 'l' )
-      {
-      int i = begin + 1;
-      for( ; i < end; ++i )
-        {
-        Character & c = character( i );
-        if( !c.guesses() ||
-            ( c.guess(0).ch != 'l' && c.guess(0).ch != 'O' ) ) break;
-        }
-      if( i >= end )
-        for( i = begin; i < end; ++i )
-        {
-        Character & c = character( i );
-        c.insert_guess( 0, (c.guess(0).ch == 'l') ? '1' : '0', c.guess(0).value + 1 );
-        }
-      }
-    }
-
   // join two adjacent single quotes into a double quote
   for( int i = 0; i < characters() - 1; ++i )
     {
@@ -483,4 +493,31 @@ void Textline::recognize2() throw()
         c.swap_guesses( 0, 1 );
       }
     }
+
+  // choose between 'B' and 'a'
+  for( int i = 0, begin = 0; i < characters(); ++i )
+    {
+    Character & c1 = character( i );
+    if( c1.guesses() )
+      {
+      unsigned char ch = c1.guess(0).ch;
+      if( std::isspace( ch ) ) { begin = i + 1 ; continue; }
+      if( c1.guesses() != 2 || ch != 'B' || c1.guess(1).ch != 'a' ) continue;
+      if( 4 * c1.height() > 5 * mean_height() ) continue;
+      for( int j = begin; j < characters(); ++j ) if( j != i )
+        {
+        Character & c2 = character( j );
+        if( c2.guesses() >= 1 )
+          {
+          unsigned char ch2 = c2.guess(0).ch;
+          if( std::isspace( ch2 ) ) break;
+          if( ( std::isalpha( ch2 ) && 5 * c1.height() < 4 * c2.height() ) ||
+              ( std::islower( ch2 ) &&
+                ( c1.height() <= c2.height() ||
+                  Ocrad::similar( c1.height(), c2.height(), 10 ) ) ) )
+            { c1.swap_guesses( 0, 1 ); break; }
+          }
+        }
+      }
+    }
   }
